import 'dart:async';
import 'dart:convert';
import 'dart:io';
import '../consts.dart';
import 'marker_data.dart';
import 'package:path_provider/path_provider.dart';

// fetchMarkerTemplate requests all marker data from the server
Future<List<Measurement>> fetchMarkerData() async {
  try {
    // Custom HTTP client
    HttpClient client = HttpClient()
      ..badCertificateCallback = // NB: temporary disable SSL certificate validation
          (X509Certificate cert, String host, int port) => true;

    // Request markers from server
    var request = await client.getUrl(Uri.parse(serverURI + mapEndpoint));
    var response = await request.close(); // Close response body at end of function

    // Parse body to JSON if request is ok
    if (response.statusCode == 200) {
      var responseBody = await response.transform(utf8.decoder).join();

      if (responseBody.isNotEmpty) {
        var jsonData = json.decode(responseBody);

        // Attempt to parse response to Measurement object only if the body
        // contains correctly formatted data
        if (jsonData != null && jsonData is List) {
          Directory appDocumentsDirectory = await getApplicationDocumentsDirectory();
          String filePath = '${appDocumentsDirectory.path}/last_update.txt';

          try { // Write most recent time of update to file
            await File(filePath).writeAsString('${DateTime.now()}', mode: FileMode.write);
            print('Update time written to file');
          } catch (error) { print('Error in writing to file: $error');}

          // Update lastUpdate variable
          lastUpdate = DateTime.now();

          return jsonData.map((data) => Measurement.fromJson(data)).toList();
        } else {
          throw Exception('Failed to parse marker data: Unexpected response format');
        }
      } else {
        throw Exception('Failed to parse marker data: Empty response body');
      }
    } else {
      throw Exception('Failed to fetch marker data: Status code ${response.statusCode}');
    }
  } catch (e) {
    throw Exception('Failed to fetch marker data: ${e.toString()}');
  }
}
